@@ -137,11 +137,12 @@ class Saml2UnscopedToken(v3.AuthConstructor):
             return response
 
         location = response.headers['location']
-        return session.request(location, method, **kwargs)
+        return session.request(location, method, authenticated=False,
+                               **kwargs)
 
     def _first(self, _list):
         if len(_list) != 1:
-            raise IndexError("Only single element list can be flatten")
+            raise IndexError("Only single element is acceptable")
         return _list[0]
 
     def _prepare_idp_saml2_request(self, saml2_authn_request):
@@ -176,7 +177,7 @@ class Saml2UnscopedToken(v3.AuthConstructor):
                    "%(sp_consumer_url)s and Identity Provider "
                    "%(identity_provider)s %(idp_consumer_url)s are not equal")
             msg = msg % {
-                'service_provider': self.saml2_token_url,
+                'service_provider': self.token_url,
                 'sp_consumer_url': sp_response_consumer_url,
                 'identity_provider': self.identity_provider,
                 'idp_consumer_url': idp_sp_response_consumer_url
@@ -207,7 +208,7 @@ class Saml2UnscopedToken(v3.AuthConstructor):
         :type session: keystoneclient.session.Session
 
         """
-        sp_response = session.get(self.saml2_token_url,
+        sp_response = session.get(self.token_url,
                                   headers=self.ECP_SP_EMPTY_REQUEST_HEADERS,
                                   authenticated=False)
 
@@ -244,7 +245,8 @@ class Saml2UnscopedToken(v3.AuthConstructor):
             self.identity_provider_url,
             headers={'Content-type': 'text/xml'},
             data=etree.tostring(idp_saml2_authn_request),
-            requests_auth=(self.username, self.password))
+            requests_auth=(self.username, self.password),
+            authenticated=False, log=False)
 
         try:
             self.saml2_idp_authn_response = etree.XML(idp_response.content)
@@ -299,7 +301,7 @@ class Saml2UnscopedToken(v3.AuthConstructor):
         self.authenticated_response = response
 
     @property
-    def saml2_token_url(self):
+    def token_url(self):
         """Return full URL where authorization data is sent."""
         values = {
             'host': self.auth_url.rstrip('/'),
@@ -409,3 +411,27 @@ class Saml2UnscopedToken(v3.AuthConstructor):
         token, token_json = self._get_unscoped_token(session, **kwargs)
         return access.AccessInfoV3(token,
                                    **token_json)
+
+
+class Saml2ScopedTokenMethod(v3.TokenMethod):
+    _method_name = 'saml2'
+
+    def get_auth_data(self, session, auth, headers, **kwargs):
+        """Build and return request body for token scoping step."""
+
+        t = super(Saml2ScopedTokenMethod, self).get_auth_data(
+            session, auth, headers, **kwargs)
+        _token_method, token = t
+        return self._method_name, token
+
+
+class Saml2ScopedToken(v3.Token):
+    """Class for scoping unscoped saml2 token."""
+
+    _auth_method_class = Saml2ScopedTokenMethod
+
+    def __init__(self, auth_url, token, **kwargs):
+        super(Saml2ScopedToken, self).__init__(auth_url, token, **kwargs)
+        if not (self.project_id or self.domain_id):
+            raise exceptions.ValidationError(
+                'Neither project nor domain specified')
